# Generated by Django 6.0.2 on 2026-02-27 07:44

import django.core.validators
import django.db.models.deletion
import django.utils.timezone
import django_email_learning.models
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="BlockedEmail",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("email", models.EmailField(max_length=254, unique=True)),
            ],
        ),
        migrations.CreateModel(
            name="Course",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=200)),
                (
                    "slug",
                    models.SlugField(
                        help_text="A short label for the course, used in URLs or email interactive actions. You can not edit it later."
                    ),
                ),
                ("description", models.TextField(blank=True, null=True)),
                ("enabled", models.BooleanField(default=False)),
                (
                    "image",
                    models.ImageField(
                        blank=True, null=True, upload_to="course_images/"
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="ImapConnection",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("salt", models.CharField(editable=False, max_length=32)),
                (
                    "server",
                    models.CharField(
                        max_length=200,
                        validators=[django_email_learning.models.is_domain_or_ip],
                    ),
                ),
                ("port", models.IntegerField(db_default=993)),
                ("email", models.EmailField(max_length=200, unique=True)),
                ("password", models.CharField(max_length=200)),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="JobExecution",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "job_name",
                    models.CharField(
                        choices=[("DELIVER_CONTENTS", "deliver_contents")],
                        max_length=200,
                    ),
                ),
                ("started_at", models.DateTimeField(auto_now_add=True)),
                ("finished_at", models.DateTimeField(blank=True, null=True)),
                (
                    "status",
                    models.CharField(
                        choices=[("RUNNING", "running"), ("COMPLETED", "completed")],
                        max_length=50,
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Learner",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("email", models.EmailField(max_length=254)),
                ("created_at", models.DateTimeField(auto_now_add=True)),
            ],
        ),
        migrations.CreateModel(
            name="Lesson",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=200)),
                ("content", models.TextField()),
            ],
        ),
        migrations.CreateModel(
            name="Organization",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=200, unique=True)),
                (
                    "logo",
                    models.ImageField(
                        blank=True, null=True, upload_to="organization_logos/"
                    ),
                ),
                ("description", models.TextField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name="Question",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("text", models.CharField(max_length=500)),
                ("priority", models.IntegerField()),
            ],
        ),
        migrations.CreateModel(
            name="Quiz",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=500)),
                (
                    "required_score",
                    models.IntegerField(
                        validators=[django.core.validators.MaxValueValidator(100)]
                    ),
                ),
                (
                    "selection_strategy",
                    models.CharField(
                        choices=[
                            ("all", "All Questions"),
                            ("random", "Random Questions"),
                        ],
                        max_length=50,
                    ),
                ),
                (
                    "deadline_days",
                    models.IntegerField(
                        help_text="Time limit to complete the quiz in days. Minimum is 1 day and maximum is 30 days.",
                        validators=[
                            django.core.validators.MinValueValidator(1),
                            django.core.validators.MaxValueValidator(30),
                        ],
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "Quizzes",
            },
        ),
        migrations.CreateModel(
            name="ApiKey",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("salt", models.CharField(editable=False, max_length=32)),
                (
                    "key",
                    models.CharField(
                        max_length=256,
                        unique=True,
                        validators=[django.core.validators.MinLengthValidator(50)],
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                (
                    "created_by",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="CourseContent",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("priority", models.IntegerField()),
                (
                    "type",
                    models.CharField(
                        choices=[("lesson", "Lesson"), ("quiz", "Quiz")], max_length=50
                    ),
                ),
                (
                    "waiting_period",
                    models.IntegerField(
                        help_text="Waiting period in seconds after previous content is sent or submited."
                    ),
                ),
                ("is_published", models.BooleanField(default=False)),
                (
                    "course",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="django_email_learning.course",
                    ),
                ),
                (
                    "lesson",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="django_email_learning.lesson",
                    ),
                ),
                (
                    "quiz",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="django_email_learning.quiz",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="ContentDelivery",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("hash_value", models.CharField(blank=True, max_length=64, null=True)),
                ("valid_until", models.DateTimeField(blank=True, null=True)),
                (
                    "course_content",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="django_email_learning.coursecontent",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="DeliverySchedule",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "time",
                    models.DateTimeField(
                        db_index=True, default=django.utils.timezone.now
                    ),
                ),
                ("link", models.URLField(blank=True, max_length=500, null=True)),
                (
                    "status",
                    models.CharField(
                        choices=[
                            (
                                django_email_learning.models.DeliveryStatus[
                                    "SCHEDULED"
                                ],
                                "Scheduled",
                            ),
                            (
                                django_email_learning.models.DeliveryStatus[
                                    "PROCESSING"
                                ],
                                "Processing",
                            ),
                            (
                                django_email_learning.models.DeliveryStatus[
                                    "DELIVERED"
                                ],
                                "Delivered",
                            ),
                            (
                                django_email_learning.models.DeliveryStatus["CANCELED"],
                                "Canceled",
                            ),
                            (
                                django_email_learning.models.DeliveryStatus["BLOCKED"],
                                "Blocked",
                            ),
                        ],
                        db_index=True,
                        default=django_email_learning.models.DeliveryStatus[
                            "SCHEDULED"
                        ],
                        max_length=50,
                    ),
                ),
                ("failed_attempts", models.IntegerField(default=0)),
                ("delivered_at", models.DateTimeField(blank=True, null=True)),
                (
                    "delivery",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="delivery_schedules",
                        to="django_email_learning.contentdelivery",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Enrollment",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("enrolled_at", models.DateTimeField(auto_now_add=True)),
                ("activated_at", models.DateTimeField(blank=True, null=True)),
                ("final_state_at", models.DateTimeField(blank=True, null=True)),
                (
                    "status",
                    models.CharField(
                        choices=[
                            (
                                django_email_learning.models.EnrollmentStatus[
                                    "UNVERIFIED"
                                ],
                                "Unverified",
                            ),
                            (
                                django_email_learning.models.EnrollmentStatus["ACTIVE"],
                                "Active",
                            ),
                            (
                                django_email_learning.models.EnrollmentStatus[
                                    "COMPLETED"
                                ],
                                "Completed",
                            ),
                            (
                                django_email_learning.models.EnrollmentStatus[
                                    "DEACTIVATED"
                                ],
                                "Deactivated",
                            ),
                        ],
                        default=django_email_learning.models.EnrollmentStatus[
                            "UNVERIFIED"
                        ],
                        max_length=50,
                    ),
                ),
                (
                    "deactivation_reason",
                    models.CharField(
                        blank=True,
                        choices=[
                            (
                                django_email_learning.models.DeactivationReason[
                                    "CANCELED"
                                ],
                                "Canceled",
                            ),
                            (
                                django_email_learning.models.DeactivationReason[
                                    "BLOCKED"
                                ],
                                "Blocked",
                            ),
                            (
                                django_email_learning.models.DeactivationReason[
                                    "FAILED"
                                ],
                                "Failed",
                            ),
                            (
                                django_email_learning.models.DeactivationReason[
                                    "INACTIVE"
                                ],
                                "Inactive",
                            ),
                        ],
                        max_length=50,
                        null=True,
                    ),
                ),
                (
                    "activation_code",
                    models.CharField(blank=True, max_length=6, null=True),
                ),
                (
                    "course",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="django_email_learning.course",
                    ),
                ),
                (
                    "learner",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="django_email_learning.learner",
                    ),
                ),
            ],
        ),
        migrations.AddField(
            model_name="contentdelivery",
            name="enrollment",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="content_deliveries",
                to="django_email_learning.enrollment",
            ),
        ),
        migrations.CreateModel(
            name="Certificate",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("issued_at", models.DateTimeField(auto_now_add=True)),
                ("name_on_certificate", models.CharField(max_length=200)),
                ("random_suffix", models.IntegerField()),
                (
                    "enrollment",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="certificate",
                        to="django_email_learning.enrollment",
                    ),
                ),
            ],
        ),
        migrations.AddField(
            model_name="course",
            name="imap_connection",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="django_email_learning.imapconnection",
            ),
        ),
        migrations.AddField(
            model_name="learner",
            name="organization",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="django_email_learning.organization",
            ),
        ),
        migrations.AddField(
            model_name="imapconnection",
            name="organization",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="django_email_learning.organization",
            ),
        ),
        migrations.AddField(
            model_name="course",
            name="organization",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="django_email_learning.organization",
            ),
        ),
        migrations.CreateModel(
            name="OrganizationUser",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "role",
                    models.CharField(
                        choices=[
                            ("admin", "Admin"),
                            ("editor", "Editor"),
                            ("viewer", "Viewer"),
                        ],
                        db_index=True,
                        max_length=50,
                    ),
                ),
                (
                    "organization",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="django_email_learning.organization",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="memberships",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Answer",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("text", models.CharField(max_length=500)),
                ("is_correct", models.BooleanField(default=False)),
                (
                    "question",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="answers",
                        to="django_email_learning.question",
                    ),
                ),
            ],
        ),
        migrations.AddField(
            model_name="question",
            name="quiz",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="questions",
                to="django_email_learning.quiz",
            ),
        ),
        migrations.CreateModel(
            name="QuizSubmission",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("score", models.IntegerField()),
                ("is_passed", models.BooleanField()),
                ("submitted_at", models.DateTimeField(auto_now_add=True)),
                (
                    "delivery",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="quiz_submissions",
                        to="django_email_learning.contentdelivery",
                    ),
                ),
            ],
        ),
        migrations.AlterUniqueTogether(
            name="contentdelivery",
            unique_together={("enrollment", "course_content")},
        ),
        migrations.AddConstraint(
            model_name="enrollment",
            constraint=models.UniqueConstraint(
                condition=models.Q(
                    ("status__in", ["unverified", "active", "completed"])
                ),
                fields=("learner", "course"),
                name="unique_active_enrollment",
            ),
        ),
        migrations.AlterUniqueTogether(
            name="learner",
            unique_together={("organization", "email")},
        ),
        migrations.AlterUniqueTogether(
            name="course",
            unique_together={("slug", "organization"), ("title", "organization")},
        ),
        migrations.AlterUniqueTogether(
            name="organizationuser",
            unique_together={("user", "organization")},
        ),
        migrations.AddConstraint(
            model_name="coursecontent",
            constraint=models.UniqueConstraint(
                condition=models.Q(("quiz__isnull", False)),
                fields=("course", "quiz"),
                name="unique_quiz_per_course",
            ),
        ),
        migrations.AddConstraint(
            model_name="coursecontent",
            constraint=models.UniqueConstraint(
                condition=models.Q(("lesson__isnull", False)),
                fields=("course", "lesson"),
                name="unique_lesson_per_course",
            ),
        ),
        migrations.AddConstraint(
            model_name="coursecontent",
            constraint=models.UniqueConstraint(
                fields=("course", "priority"), name="unique_priority_per_course"
            ),
        ),
    ]



from django_email_learning.services.command_models.abstract_command import (
    AbstractCommand,
)
from django_email_learning.models import (
    Lesson,
    CourseContent,
    Enrollment,
    EnrollmentStatus,
)
from django_email_learning.services.email_sender_service import EmailSenderService
from django_email_learning.services.metrics_service import MetricsService
from django.core.mail import EmailMultiAlternatives
from django.template.loader import render_to_string
from typing import Literal


class LessonNotFoundError(Exception):
    pass


class SendLessonCommand(AbstractCommand):
    command_name: Literal["send_lesson"] = "send_lesson"
    content_id: int
    email: str

    def execute(self) -> None:
        metric_service = MetricsService()
        content = CourseContent.objects.get(id=self.content_id)
        if not content.lesson:
            raise LessonNotFoundError(
                f"CourseContent with ID {self.content_id} has no associated lesson"
            )
        self.logger.info(
            f"Sending lesson with ID {content.lesson.id} to email {self.email}"
        )

        try:
            lesson = Lesson.objects.get(id=content.lesson.id)
        except Lesson.DoesNotExist:
            raise LessonNotFoundError(f"Lesson with ID {content.lesson.id} not found")

        subject = lesson.title
        enrollment = Enrollment.objects.filter(
            course=content.course,
            learner__email=self.email,
            status=EnrollmentStatus.ACTIVE,
        ).first()
        if not enrollment:
            progress = 0
        else:
            progress = enrollment.progress_percentage
        next_content = content.get_next()
        context = {
            "lesson": lesson,
            "unsubscribe_link": content.course.generate_unsubscribe_link(self.email),
            "progress": progress,
            "next_content": next_content,
        }
        payload = render_to_string("emails/lesson.txt", context)

        email_service = EmailSenderService()

        email_message = EmailMultiAlternatives(
            subject=subject,
            body=payload,
            from_email=email_service.from_email,
            to=[self.email],
        )
        email_message.attach_alternative(
            render_to_string("emails/lesson.html", context), "text/html"
        )

        email_service.send(email_message)
        metric_service.lesson_sent(
            course_slug=content.course.slug,
            organization_id=content.course.organization.id,
            lesson_id=lesson.id,
        )



from django_email_learning.services.command_models.abstract_command import (
    AbstractCommand,
)
from django_email_learning.services.command_models.enroll_command import EnrollCommand
from django_email_learning.services.command_models.verify_enrollment_command import (
    VerifyEnrollmentCommand,
)
from django_email_learning.models import Course, Enrollment, EnrollmentStatus
from google_auth_oauthlib.flow import Flow  #  type: ignore
from typing import Literal
from django.conf import settings
from urllib import error, parse, request
from django.urls import reverse
import json


DJANGO_EMAIL_LEARNING_SETTINGS: dict = getattr(settings, "DJANGO_EMAIL_LEARNING", {})


class EnrollFromGoogleDirectoryCommand(AbstractCommand):
    command_name: Literal[
        "enroll_from_google_directory"
    ] = "enroll_from_google_directory"
    code: str | None = None
    course_id: int
    state: str | None = None
    code_verifier: str | None = None

    def _build_flow(self) -> Flow:
        flow = Flow.from_client_config(
            client_config={
                "web": {
                    "client_id": DJANGO_EMAIL_LEARNING_SETTINGS.get(
                        "GOOGLE_OAUTH_CLIENT_ID"
                    ),
                    "client_secret": DJANGO_EMAIL_LEARNING_SETTINGS.get(
                        "GOOGLE_OAUTH_CLIENT_SECRET"
                    ),
                    "auth_uri": "https://accounts.google.com/o/oauth2/auth",
                    "token_uri": "https://oauth2.googleapis.com/token",
                }
            },
            scopes=["https://www.googleapis.com/auth/admin.directory.user.readonly"],
            state=self.state,
        )
        flow.redirect_uri = DJANGO_EMAIL_LEARNING_SETTINGS.get(
            "SITE_BASE_URL", "http://localhost:8000"
        ) + reverse("django_email_learning:oauth_integrations:redirect_view")
        return flow

    def get_authorization_url(self, state: str) -> str:
        self.state = state
        flow = self._build_flow()
        authorization_url, _ = flow.authorization_url(
            access_type="offline",
            include_granted_scopes="true",
            prompt="consent",
            code_challenge_method="S256",
        )
        self.code_verifier = flow.code_verifier
        return authorization_url

    def execute(self) -> None:
        if not self.code or not self.state:
            raise ValueError(
                "Authorization code and state are required to enroll from Google Directory"
            )
        if not self.code_verifier:
            raise ValueError(
                "Code verifier is required to enroll from Google Directory"
            )

        course = Course.objects.get(id=self.course_id)

        flow = self._build_flow()
        flow.code_verifier = self.code_verifier

        flow.fetch_token(code=self.code)
        credentials = flow.credentials

        access_token = credentials.token
        if not access_token:
            raise ValueError("Unable to retrieve access token from Google OAuth flow")

        users_endpoint = "https://admin.googleapis.com/admin/directory/v1/users"
        page_token: str | None = None
        enrolled_count = 0
        failed_count = 0

        while True:
            query = {
                "customer": "my_customer",
                "orderBy": "email",
                "maxResults": 500,
            }
            if page_token:
                query["pageToken"] = page_token
            url = f"{users_endpoint}?{parse.urlencode(query)}"

            req = request.Request(
                url,
                headers={
                    "Authorization": f"Bearer {access_token}",
                    "Accept": "application/json",
                },
                method="GET",
            )

            try:
                with request.urlopen(req) as response:
                    payload = json.loads(response.read().decode("utf-8"))
            except error.HTTPError as e:
                details = e.read().decode("utf-8") if e.fp else str(e)
                raise ValueError(
                    f"Google Directory API request failed: {details}"
                ) from e

            users = payload.get("users", [])
            for user in users:
                email = user.get("primaryEmail")
                if not email:
                    continue
                try:
                    EnrollCommand(
                        email=email,
                        course_slug=course.slug,
                        organization_id=course.organization_id,
                        no_verification=True,
                    ).execute()
                    enrolled_count += 1
                    enrollment = Enrollment.objects.get(
                        learner__email=email,
                        course_id=self.course_id,
                        status=EnrollmentStatus.UNVERIFIED,
                    )
                    VerifyEnrollmentCommand(
                        enrollment_id=enrollment.id,
                        verification_code=enrollment.activation_code,  # type: ignore[arg-type]
                    ).execute()
                except Exception as e:  # noqa: BLE001
                    failed_count += 1
                    self.logger.warning(
                        "Failed to enroll %s from Google Directory: %s", email, str(e)
                    )

            page_token = payload.get("nextPageToken")
            if not page_token:
                break

        self.logger.info(
            "Google Directory import finished for course_id=%s: enrolled=%s failed=%s",
            self.course_id,
            enrolled_count,
            failed_count,
        )



from pydantic import BaseModel, field_validator


class QuestionResponse(BaseModel):
    id: int
    answers: set[int]


class QuizSubmissionRequest(BaseModel):
    answers: list[QuestionResponse]
    token: str

    @field_validator("answers")
    def validate_answers(cls, v: list[QuestionResponse]) -> list[QuestionResponse]:
        question_ids = []
        for response in v:
            if response.id in question_ids:
                raise ValueError(f"Duplicate question ID found: {response.id}")
            question_ids.append(response.id)
        return v



from functools import wraps
from django.http import JsonResponse
from django_email_learning.models import OrganizationUser
from django_email_learning.apps import PLATFORM_ADMIN_GROUP_NAME
from django_email_learning.services.jwt_service import decode_jwt, InvalidTokenException
from django_email_learning.models import ApiKey
import typing


def is_platform_admin() -> typing.Callable:
    def decorator(view_func: typing.Callable) -> typing.Callable:
        @wraps(view_func)
        def _wrapped_view(request, *view_args, **view_kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
            if not request.user.is_authenticated:
                return JsonResponse({"error": "Unauthorized"}, status=401)

            if (
                not request.user.is_superuser
                and not request.user.groups.filter(
                    name=PLATFORM_ADMIN_GROUP_NAME
                ).exists()
            ):
                return JsonResponse({"error": "Forbidden"}, status=403)
            return view_func(request, *view_args, **view_kwargs)

        return _wrapped_view

    return decorator


def accessible_for(roles: set[str]) -> typing.Callable:
    def decorator(view_func: typing.Callable) -> typing.Callable:
        @wraps(view_func)
        def _wrapped_view(request, *view_args, **view_kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
            user = request.user
            if not user.is_authenticated:
                return JsonResponse({"error": "Unauthorized"}, status=401)

            if not user.is_superuser:
                has_access = OrganizationUser.objects.filter(  # type: ignore[misc]
                    user=user,
                    organization_id=view_kwargs.get("organization_id"),
                    role__in=roles,
                ).exists()
                if not has_access:
                    return JsonResponse({"error": "Forbidden"}, status=403)
            return view_func(request, *view_args, **view_kwargs)

        return _wrapped_view

    return decorator


def is_an_organization_member(only_admin: bool = False) -> typing.Callable:
    def decorator(view_func: typing.Callable) -> typing.Callable:
        @wraps(view_func)
        def _wrapped_view(request, *view_args, **view_kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
            user = request.user
            if not user.is_authenticated:
                return JsonResponse({"error": "Unauthorized"}, status=401)

            if not user.is_superuser:
                qs = OrganizationUser.objects.filter(  # type: ignore[misc]
                    user=user
                )
                if only_admin:
                    qs = qs.filter(role="admin")
                has_access = qs.exists()
                if not has_access:
                    return JsonResponse({"error": "Forbidden"}, status=403)
            return view_func(request, *view_args, **view_kwargs)

        return _wrapped_view

    return decorator


def check_api_key() -> typing.Callable:
    def decorator(view_func: typing.Callable) -> typing.Callable:
        @wraps(view_func)
        def _wrapped_view(request, *view_args, **view_kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
            authorization_header = request.headers.get("Authorization")
            if not authorization_header:
                return JsonResponse(
                    {"error": "Authorization header missing"}, status=401
                )
            authorization_header_parts = authorization_header.split(" ")
            if (
                len(authorization_header_parts) != 2
                or authorization_header_parts[0] != "Bearer"
            ):
                return JsonResponse(
                    {
                        "error": "Invalid Authorization header format. Expected: Bearer <API_KEY>"
                    },
                    status=401,
                )
            api_key = authorization_header_parts[1]
            try:
                key_data = decode_jwt(api_key)
                possible_keys = ApiKey.objects.filter(salt=key_data["salt"])
                key_matched = False
                for possible_key in possible_keys:
                    key_value = possible_key.decrypt_password(possible_key.key)
                    if key_value == key_data["key"]:
                        key_matched = True
                        break
                if not key_matched:
                    return JsonResponse({"error": "Invalid API key"}, status=401)
            except InvalidTokenException:
                return JsonResponse({"error": "Invalid Json Web Token"}, status=401)
            except KeyError:
                return JsonResponse(
                    {"error": "Json Web Token missing required fields"}, status=401
                )

            try:
                api_key = ApiKey.objects.get(salt=key_data["salt"])
            except ApiKey.DoesNotExist:
                return JsonResponse({"error": "Invalid API key"}, status=401)

            return view_func(request, *view_args, **view_kwargs)

        return _wrapped_view

    return decorator



from django.utils.decorators import method_decorator
from django_email_learning.decorators import accessible_for
from django.http import JsonResponse
from django.views import View
from pydantic import ValidationError
from .serializers import EditTextRequest
import json


@method_decorator(accessible_for(roles={"admin", "editor"}), name="post")
class EditTextView(View):
    def post(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        payload = json.loads(request.body)
        try:
            serializer = EditTextRequest.model_validate(payload)
            input = serializer.input
            model = serializer.model
            ai_adapter = model.adapter_class()
            edited_text = ai_adapter.edit_text(input, model.model_name)
            return JsonResponse({"edited_text": edited_text})
        except ValidationError as e:
            return JsonResponse({"error": str(e)}, status=400)



from django_email_learning.ports.metric_recorder_protocol import MetricRecorderProtocol
from django_email_learning.models import DeactivationReason
import logging

logger = logging.getLogger(__name__)


class LogBasedMetricRecorder(MetricRecorderProtocol):
    def user_enrolled_in_course(self, course_slug: str, organization_id: int) -> None:
        logger.info(
            "User enrolled",
            extra={
                "metric": "user_enrolled",
                "course_slug": course_slug,
                "organization_id": organization_id,
            },
        )

    def quiz_sent(self, course_slug: str, organization_id: int, quiz_id: int) -> None:
        logger.info(
            "Quiz sent",
            extra={
                "metric": "quiz_sent",
                "course_slug": course_slug,
                "organization_id": organization_id,
                "quiz_id": quiz_id,
            },
        )

    def lesson_sent(
        self, course_slug: str, organization_id: int, lesson_id: int
    ) -> None:
        logger.info(
            "Lesson sent",
            extra={
                "metric": "lesson_sent",
                "course_slug": course_slug,
                "organization_id": organization_id,
                "lesson_id": lesson_id,
            },
        )

    def user_enrollment_activated(self, course_slug: str, organization_id: int) -> None:
        logger.info(
            "User enrollment activated",
            extra={
                "metric": "user_enrollment_activated",
                "course_slug": course_slug,
                "organization_id": organization_id,
            },
        )

    def user_enrollment_deactivated(
        self, course_slug: str, organization_id: int, reason: DeactivationReason
    ) -> None:
        logger.info(
            "User enrollment deactivated",
            extra={
                "metric": "user_enrollment_deactivated",
                "course_slug": course_slug,
                "organization_id": organization_id,
                "reason": reason,
            },
        )

    def user_completed_course(self, course_slug: str, organization_id: int) -> None:
        logger.info(
            "User completed course",
            extra={
                "metric": "user_completed_course",
                "course_slug": course_slug,
                "organization_id": organization_id,
            },
        )

    def quiz_submitted(
        self, course_slug: str, organization_id: int, quiz_id: int, is_passed: bool
    ) -> None:
        logger.info(
            "Quiz submitted",
            extra={
                "metric": "quiz_submitted",
                "course_slug": course_slug,
                "organization_id": organization_id,
                "quiz_id": quiz_id,
                "is_passed": is_passed,
            },
        )

    def method_executed(self, method_name: str, execution_time: int) -> None:
        logger.info(
            "Method executed",
            extra={
                "metric": "method_executed",
                "method_name": method_name,
                "execution_time": execution_time,
            },
        )

    def delivery_schedule_blocked(self, content_id: int) -> None:
        logger.info(
            "Delivery schedule blocked",
            extra={
                "metric": "delivery_schedule_blocked",
                "content_id": content_id,
            },
        )



from django_email_learning.ports.delivery_queue_protocol import DeliveryQueueProtocol
from django_email_learning.models import DeliverySchedule, DeliveryStatus
from django_email_learning.services.command_models.send_lesson_command import (
    SendLessonCommand,
    LessonNotFoundError,
)
from django_email_learning.services.command_models.send_quiz_command import (
    SendQuizCommand,
    QuizNotFoundError,
)
from django_email_learning.services.metrics_service import MetricsService
from django_email_learning.models import JobExecution, JobName, JobStatus
from django.utils.module_loading import import_string
from django.conf import settings
from django.utils import timezone
import logging
import datetime


logger = logging.getLogger(__name__)
METRIC_SERVICE = MetricsService()


class DeliverContentsJob:
    def __init__(self) -> None:
        self.delivery_queue: DeliveryQueueProtocol = self.get_delivery_queue()

    def run(self) -> None:
        if JobExecution.objects.filter(
            job_name=JobName.DELIVER_CONTENTS.value,
            status=JobStatus.RUNNING.value,
        ).exists():
            logger.warning(
                "Another instance of DeliverContentsJob is already running. Exiting this run."
            )
            return
        job_execution = JobExecution.objects.create(
            job_name=JobName.DELIVER_CONTENTS.value,
            status=JobStatus.RUNNING.value,
            started_at=timezone.now(),
        )
        should_check_next = True
        while should_check_next:
            delivery_schedule = self.delivery_queue.next_task()
            if delivery_schedule is None:
                should_check_next = False
                job_execution.status = JobStatus.COMPLETED.value
                job_execution.finished_at = timezone.now()
                job_execution.save()
            else:
                try:
                    self.process_delivery(delivery_schedule)
                except Exception as e:
                    # Unhandled exception during delivery processing should not crash the job.
                    # We log the error and mark the delivery as blocked to prevent further attempts until manual intervention.
                    delivery_schedule.status = DeliveryStatus.BLOCKED
                    delivery_schedule.save()
                    METRIC_SERVICE.delivery_schedule_blocked(
                        delivery_schedule.delivery.course_content.id
                    )
                    logger.exception(
                        f"Error processing delivery schedule: {str(e)}. Continuing with next task."
                    )

    def get_delivery_queue(self) -> DeliveryQueueProtocol:
        DJANGO_EMAIL_LEARNING_SETTINGS: dict = getattr(
            settings, "DJANGO_EMAIL_LEARNING", {}
        )
        try:
            configured_delivery_queue = import_string(
                DJANGO_EMAIL_LEARNING_SETTINGS["DELIVERY_QUEUE"]
            )
            return (
                configured_delivery_queue()
                if isinstance(configured_delivery_queue, type)
                else configured_delivery_queue
            )
        except KeyError:
            from django_email_learning.services.defaults.database_delivery_queue import (
                DatabaseDeliveryQueue,
            )

            return DatabaseDeliveryQueue()

    def process_delivery(self, delivery_schedule: DeliverySchedule) -> None:
        course_content = delivery_schedule.delivery.course_content
        if not course_content.is_published:
            logger.warning(
                f"CourseCcontent {course_content.id} is not published. Canceling the delivery."
            )
            delivery_schedule.status = DeliveryStatus.CANCELED
            delivery_schedule.save()
            return

        if course_content.type == "lesson":
            is_delivered = self.send_lesson_content(delivery_schedule)
            if is_delivered:
                logger.info(
                    f"Lesson content delivered for DeliverySchedule ID {delivery_schedule.id}. Scheduling next content."
                )
                next_delivery = delivery_schedule.delivery.schedule_next_delivery()
                if next_delivery:
                    logger.info(
                        f"Scheduled next delivery {next_delivery.id} for enrollment {delivery_schedule.delivery.enrollment.id}"
                    )
                else:
                    logger.info(
                        f"No more content to schedule for enrollment {delivery_schedule.delivery.enrollment.id}"
                    )
                    # TODO: if the sent content was the last in the course, consider marking the enrollment as completed.
                    delivery_schedule.delivery.enrollment.graduate()

        elif course_content.type == "quiz":
            is_delivered = self.send_quiz_content(delivery_schedule)

            # For quiz we don't schedule next content automatically, because the scheduling should be done after quiz completion.
            if is_delivered:
                logger.info(
                    f"Quiz content delivered for DeliverySchedule ID {delivery_schedule.id}. Next content scheduling is deferred until quiz completion."
                )

    def send_lesson_content(self, delivery_schedule: DeliverySchedule) -> bool:
        if not delivery_schedule.delivery.course_content.lesson:
            delivery_schedule.status = DeliveryStatus.CANCELED
            delivery_schedule.save()
            logger.error(
                f"DeliverySchedule ID {delivery_schedule.id} has no associated lesson. Canceling the delivery."
            )
            return False

        try:
            command = SendLessonCommand(
                content_id=delivery_schedule.delivery.course_content.id,
                email=delivery_schedule.delivery.enrollment.learner.email,
            )
            command.execute()
            delivery_schedule.status = DeliveryStatus.DELIVERED
            delivery_schedule.save()
            return True

        except LessonNotFoundError:
            logger.error(
                f"Lesson with ID {delivery_schedule.delivery.course_content.lesson.id} not found. Canceling the delivery."
            )
            delivery_schedule.status = DeliveryStatus.CANCELED
            delivery_schedule.save()
        except Exception as e:
            logger.exception(
                f"Failed to send lesson content for DeliverySchedule ID {delivery_schedule.id}: {str(e)}"
            )
            self.handle_failed_delivery(delivery_schedule)
        return False

    def send_quiz_content(self, delivery_schedule: DeliverySchedule) -> bool:
        if not delivery_schedule.delivery.course_content.quiz:
            delivery_schedule.status = DeliveryStatus.CANCELED
            delivery_schedule.save()
            logger.error(
                f"DeliverySchedule ID {delivery_schedule.id} has no associated quiz. Canceling the delivery."
            )
            return False

        try:
            if not delivery_schedule.link:
                link = delivery_schedule.generate_link()
                delivery_schedule.link = link
                delivery_schedule.save()

            command = SendQuizCommand(
                content_id=delivery_schedule.delivery.course_content.id,
                email=delivery_schedule.delivery.enrollment.learner.email,
                link=delivery_schedule.link,
            )
            command.execute()
            delivery_schedule.status = DeliveryStatus.DELIVERED
            delivery_schedule.save()

            return True
        except QuizNotFoundError:
            logger.error(
                f"Quiz with ID {delivery_schedule.delivery.course_content.quiz.id} not found. Canceling the delivery."
            )
            delivery_schedule.status = DeliveryStatus.CANCELED
            delivery_schedule.save()
        except Exception as e:
            logger.exception(
                f"Failed to send quiz content for DeliverySchedule ID {delivery_schedule.id}: {str(e)}"
            )
            self.handle_failed_delivery(delivery_schedule)
        return False

    def handle_failed_delivery(self, delivery_schedule: DeliverySchedule) -> None:
        # TODO: Implement custome metric logging for blocked deliveries and failed attempts.
        """Handle a failed delivery by rescheduling or blocking it."""
        if delivery_schedule.failed_attempts >= 3:
            logger.error(
                f"DeliverySchedule ID {delivery_schedule.id} has reached maximum retry attempts. Blocking the delivery."
            )
            delivery_schedule.status = DeliveryStatus.BLOCKED
            delivery_schedule.save()
            METRIC_SERVICE.delivery_schedule_blocked(
                delivery_schedule.delivery.course_content.id
            )
        else:
            delivery_schedule.time += datetime.timedelta(minutes=60)
            delivery_schedule.failed_attempts += 1
            delivery_schedule.status = DeliveryStatus.SCHEDULED
            delivery_schedule.save()



from enum import StrEnum
from pydantic import BaseModel


class Display(StrEnum):
    INLINE = "inline"
    BLOCK = "block"
    INLINE_BLOCK = "inline-block"
    NONE = "none"


class WebComponent(BaseModel):
    script_url: str | None = None
    style_url: str | None = None
    html: str
    container_display: Display = Display.INLINE_BLOCK



from django_email_learning.services.command_models.abstract_command import (
    AbstractCommand,
)
from django_email_learning.services.command_models.exceptions.invalid_course_slug_error import (
    InvalidCourseSlugError,
)
from django_email_learning.services.command_models.exceptions.enrollment_already_exists_error import (
    EnrollmentAlreadyExistsError,
)
from django_email_learning.services.command_models.exceptions.blocked_email_error import (
    BlockedEmailError,
)
from django_email_learning.models import (
    BlockedEmail,
    Learner,
    Enrollment,
    Course,
    EnrollmentStatus,
)
from django_email_learning.services.utils import mask_email
from django_email_learning.services.email_sender_service import EmailSenderService
from django_email_learning.services import jwt_service
from django_email_learning.services.metrics_service import MetricsService
from django.core.mail import EmailMultiAlternatives
from django.template.loader import render_to_string
from django.utils.translation import gettext as _
from django.conf import settings
from django.urls import reverse
from typing import Literal


class EnrollCommand(AbstractCommand):
    command_name: Literal["enroll"] = "enroll"
    email: str
    course_slug: str
    organization_id: int
    no_verification: bool = False

    def execute(self) -> None:
        metric_service = MetricsService()
        # Check if the email is blocked
        if BlockedEmail.objects.filter(email=self.email).exists():
            self.logger.info(
                f"Enrollment Rejected: {mask_email(self.email)} is blocked"
            )
            raise BlockedEmailError(f"The email {mask_email(self.email)} is blocked.")

        # Check if Learner with the email exists, if not create one
        learner, created = Learner.objects.get_or_create(
            email=self.email, organization_id=self.organization_id
        )
        if created:
            self.logger.info(
                f"Created new Learner for email: {mask_email(self.email)}. Learner ID: {learner.id}"
            )

        try:
            course = Course.objects.get(
                slug=self.course_slug,
                organization_id=self.organization_id,
                enabled=True,
            )
        except Course.DoesNotExist:
            self.logger.error(
                f"Enrollment Failed: Invalid course slug '{self.course_slug}' for organization ID {self.organization_id}"
            )
            raise InvalidCourseSlugError(
                f"Course with slug '{self.course_slug}' does not exist or is not enabled for organization ID {self.organization_id}"
            )

        # Check if an enrollment already exists
        if (
            Enrollment.objects.filter(learner=learner, course=course)
            .exclude(status=EnrollmentStatus.DEACTIVATED)
            .exists()
        ):
            self.logger.info(
                f"Enrollment Skipped: Learner ID {learner.id} is already enrolled in course '{self.course_slug}'"
            )
            raise EnrollmentAlreadyExistsError(
                f"Learner with email {mask_email(self.email)} is already enrolled in course '{self.course_slug}'"
            )

        # Create the enrollment
        enrollment = Enrollment.objects.create(
            learner=learner, course=course, status=EnrollmentStatus.UNVERIFIED
        )

        self.logger.info(
            f"Enrollment Successful: Learner ID {learner.id} enrolled in course '{self.course_slug}'. Enrollment ID: {enrollment.id}"
        )

        metric_service.user_enrolled_in_course(self.course_slug, self.organization_id)

        if self.no_verification:
            self.logger.info(
                f"Verification email skipped for Enrollment ID: {enrollment.id} as per command parameter"
            )
            return

        # Send verification email

        token = jwt_service.generate_jwt(
            {
                "verification_code": enrollment.activation_code,
                "enrollment_id": enrollment.id,
            }
        )

        verification_relative_path = (
            reverse("django_email_learning:personalised:verify_enrollment")
            + f"?token={token}"
        )
        DJANGO_EMAIL_LEARNING_SETTINGS: dict = getattr(
            settings, "DJANGO_EMAIL_LEARNING", {}
        )
        verification_link = (
            DJANGO_EMAIL_LEARNING_SETTINGS["SITE_BASE_URL"] + verification_relative_path
        )

        template_context = {
            "course_title": course.title,
            "verification_link": verification_link,
            "verification_code": enrollment.activation_code,
            "organization_name": course.organization.name,
            "support_imap_interface": course.imap_connection is not None,
            "imap_email_address": course.imap_connection.email
            if course.imap_connection
            else None,
        }
        email_service = EmailSenderService()
        subject = _("Verify your enrollment")
        body = render_to_string(
            "emails/enrollment_verification.txt",
            template_context,
        )

        to_emails = [self.email]

        html_content = render_to_string(
            "emails/enrollment_verification.html",
            template_context,
        )

        # TODO: Add AMP content/type to activate directly in email clients that support it

        email = EmailMultiAlternatives(
            subject=subject,
            body=body,
            from_email=email_service.from_email,
            to=to_emails,
        )
        email.attach_alternative(html_content, "text/html")
        email_service.send(email)

        self.logger.info(
            f"Verification email sent to {mask_email(self.email)} for Enrollment ID: {enrollment.id}"
        )



from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.utils.module_loading import import_string


class EmailSenderService:
    def __init__(self) -> None:
        DJANGO_EMAIL_LEARNING_SETTINGS: dict = getattr(
            settings, "DJANGO_EMAIL_LEARNING", {}
        )
        try:
            configured_email_sender = import_string(
                DJANGO_EMAIL_LEARNING_SETTINGS["EMAIL_SENDER"]
            )
            self.email_sender = (
                configured_email_sender()
                if isinstance(configured_email_sender, type)
                else configured_email_sender
            )
        except KeyError:
            from django_email_learning.services.defaults.email_sender import (
                DjangoEmailSender,
            )

            self.email_sender = DjangoEmailSender()

        try:
            self.from_email = DJANGO_EMAIL_LEARNING_SETTINGS["FROM_EMAIL"]
        except (AttributeError, KeyError):
            try:
                self.from_email = settings.DEFAULT_FROM_EMAIL
            except AttributeError:
                self.from_email = ""
        if not self.from_email:
            raise ValueError(
                "Either set DJANGO_EMAIL_LEARNING['FROM_EMAIL'] or DEFAULT_FROM_EMAIL."
            )

    def send(self, email: EmailMultiAlternatives) -> None:
        self.email_sender.send_email(email)



from django.urls import path
from django.views.generic import RedirectView
from django_email_learning.platform.views import (
    CourseView,
    Courses,
    Organizations,
    SingleOrganization,
    Learners,
    ApiKeys,
)

app_name = "django_email_learning"

urlpatterns = [
    path("courses/", Courses.as_view(), name="courses_view"),
    path("courses/<int:course_id>/", CourseView.as_view(), name="course_detail_view"),
    path("organizations/", Organizations.as_view(), name="organizations_view"),
    path(
        "organizations/<int:organization_id>/",
        SingleOrganization.as_view(),
        name="organization_detail_view",
    ),
    path("learners/", Learners.as_view(), name="learners_view"),
    path("settings/api_keys/", ApiKeys.as_view(), name="api_keys_view"),
    path(
        "",
        RedirectView.as_view(
            pattern_name="django_email_learning:platform:courses_view"
        ),
        name="root",
    ),
]



from django.core.management.base import BaseCommand
from django_email_learning.jobs.deliver_contents_job import DeliverContentsJob
from django.core.management.base import CommandParser
import logging


class Command(BaseCommand):
    help = "Run the content delivery job to process scheduled content deliveries"

    def add_arguments(self, parser: CommandParser) -> None:
        parser.add_argument(
            "--verbose",
            action="store_true",
            help="Enable verbose logging output",
        )

    def handle(self, *args, **options) -> None:  # type: ignore[no-untyped-def]
        # Configure logging based on verbosity
        if options["verbose"]:
            logging.basicConfig(
                level=logging.DEBUG,
                format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
            )
        else:
            logging.basicConfig(
                level=logging.INFO, format="%(asctime)s - %(levelname)s - %(message)s"
            )

        logger = logging.getLogger(__name__)

        try:
            self.stdout.write("Starting content delivery job...")
            logger.info("Starting DeliverContentsJob")

            job = DeliverContentsJob()
            job.run()

            self.stdout.write(
                self.style.SUCCESS("Content delivery job completed successfully")
            )
            logger.info("DeliverContentsJob completed successfully")

        except KeyboardInterrupt:
            self.stdout.write(
                self.style.WARNING("Content delivery job interrupted by user")
            )
            logger.warning("DeliverContentsJob interrupted by user")

        except Exception as e:
            self.stdout.write(
                self.style.ERROR(f"Content delivery job failed: {str(e)}")
            )
            logger.error(f"DeliverContentsJob failed: {str(e)}", exc_info=True)
            raise



from pydantic import BaseModel, Field
from django_email_learning.services.command_models.enroll_command import EnrollCommand
from django_email_learning.services.command_models.unsubscribe_command import (
    UnsubscribeCommand,
)
from django_email_learning.services.command_models.enroll_from_google_directory_command import (
    EnrollFromGoogleDirectoryCommand,
)


class CommandRequest(BaseModel):
    command: EnrollCommand | UnsubscribeCommand | EnrollFromGoogleDirectoryCommand = (
        Field(..., discriminator="command_name")
    )



def mask_email(email_address: str) -> str:
    """Mask email address for logging privacy."""
    try:
        username, domain = email_address.split("@")
        masked_username = username[0] + "***"
        return f"{masked_username}@{domain}"
    except ValueError:
        return "***@***"



from django.urls import path
from django_email_learning.public.api.views import EnrollView

app_name = "django_email_learning"

urlpatterns = [
    path("enroll/", EnrollView.as_view(), name="enroll"),
]



from enum import Enum
from .open_ai_adapter import OpenAiAdapter
from .ai_service_protocol import AiServiceProtocol


class LanguageModel(Enum):
    """Supported AI text-editing models.

    Each enum value maps to:

    - ``model_name``: The provider model identifier sent to the AI API.
    - ``adapter_class``: The adapter that knows how to call that provider.

    The project currently ships with OpenAI-backed models only.

    .. note::
       These models are used when
       ``DJANGO_EMAIL_LEARNING['AI']['TEXT_EDITING_MODEL']`` is configured.
       AI configuration is optional and only required when using AI editing
       features.
    """

    GPT_4O_MINI = ("gpt-4o-mini", OpenAiAdapter)
    """OpenAI GPT-4o mini model (balanced quality and speed)."""

    GPT_5_NANO = ("gpt-5-nano", OpenAiAdapter)
    """OpenAI GPT-5 nano model (smallest and fastest GPT-5 variant)."""

    GPT_5_MINI = ("gpt-5-mini", OpenAiAdapter)
    """OpenAI GPT-5 mini model (higher quality than nano, still efficient)."""

    def __init__(self, model_name: str, adapter_class: type[AiServiceProtocol]) -> None:
        """Attach provider metadata to each model enum value.

        :param model_name: External model name used by the AI provider API.
        :param adapter_class: Adapter implementing :class:`AiServiceProtocol`.
        """
        self.model_name = model_name
        self.adapter_class = adapter_class



from django.urls import path, include
from django.apps import apps
from django_email_learning.platform.api import urls as api_urls
from django_email_learning.platform import urls as platform_urls
from django_email_learning.personalised.api import urls as personalised_api_urls
from django_email_learning.personalised import urls as personalised_urls
from django_email_learning.public.api import urls as public_api_urls
from django_email_learning.public import urls as public_urls
from django_email_learning.jobs.api import urls as jobs_api_urls

app_name = "django_email_learning"

urlpatterns = [
    path("api/platform/", include(api_urls, namespace="api_platform")),
    path(
        "api/personalised/",
        include(personalised_api_urls, namespace="api_personalised"),
    ),
    path("api/public/", include(public_api_urls, namespace="api_public")),
    path("platform/", include(platform_urls, namespace="platform")),
    path("public/", include(public_urls, namespace="public")),
    path("my/", include(personalised_urls, namespace="personalised")),
    path("api/jobs/", include(jobs_api_urls, namespace="api_jobs")),
]

if apps.is_installed("django_email_learning.oauth_integrations"):
    urlpatterns += [
        path(
            "oauth/",
            include(
                "django_email_learning.oauth_integrations.urls",
                namespace="oauth_integrations",
            ),
        ),
    ]

if apps.is_installed("django_email_learning.ai"):
    urlpatterns += [
        path(
            "api/ai/",
            include("django_email_learning.ai.urls", namespace="api_ai"),
        ),
    ]



from django.views import View
from django_email_learning.decorators import check_api_key
from django_email_learning.jobs.deliver_contents_job import DeliverContentsJob
from django.utils.decorators import method_decorator
from django.http import JsonResponse


@method_decorator(check_api_key(), name="get")
class DeliverContentsJobView(View):
    def get(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        job = DeliverContentsJob()
        job.run()
        return JsonResponse({"status": "DeliverContentsJob triggered"}, status=202)



from pydantic import BaseModel, Field
from .language_models import LanguageModel


class EditTextRequest(BaseModel):
    input: str = Field(
        min_length=40,
        max_length=500,
        description="The input to be edited by the AI model.",
    )
    model: LanguageModel = Field(default=LanguageModel.GPT_5_NANO)



from django_email_learning.services.command_models.abstract_command import (
    AbstractCommand,
)
from django_email_learning.models import Quiz, CourseContent
from django_email_learning.services.email_sender_service import EmailSenderService
from django_email_learning.services.metrics_service import MetricsService
from django.core.mail import EmailMultiAlternatives
from django.template.loader import render_to_string
from typing import Literal


class QuizNotFoundError(Exception):
    pass


class SendQuizCommand(AbstractCommand):
    command_name: Literal["send_quiz"] = "send_quiz"
    link: str
    email: str
    content_id: int

    def execute(self) -> None:
        metric_service = MetricsService()
        content = CourseContent.objects.get(id=self.content_id)
        if not content.quiz:
            raise QuizNotFoundError(
                f"CourseContent with ID {self.content_id} has no associated quiz"
            )
        self.logger.info(
            f"Sending quiz with ID {content.quiz.id} to email {self.email}"
        )

        try:
            quiz = Quiz.objects.get(id=content.quiz.id)
        except Quiz.DoesNotExist:
            raise QuizNotFoundError(f"Quiz with ID {content.quiz.id} not found")

        subject = quiz.title
        context = {
            "quiz": quiz,
            "link": self.link,
            "unsubscribe_link": content.course.generate_unsubscribe_link(self.email),
        }
        payload = render_to_string("emails/quiz.txt", context)

        email_service = EmailSenderService()
        email_message = EmailMultiAlternatives(
            subject=subject,
            body=payload,
            from_email=email_service.from_email,
            to=[self.email],
        )
        email_message.attach_alternative(
            render_to_string("emails/quiz.html", context), "text/html"
        )

        email_service.send(email_message)
        metric_service.quiz_sent(
            course_slug=content.course.slug,
            organization_id=content.course.organization.id,
            quiz_id=quiz.id,
        )



from typing import Literal
from django_email_learning.models import (
    Course,
    Enrollment,
    DeliveryStatus,
    Learner,
    EnrollmentStatus,
    DeactivationReason,
)
from django.utils import timezone
from django_email_learning.services.metrics_service import MetricsService
from django_email_learning.services.command_models.abstract_command import (
    AbstractCommand,
)
from django_email_learning.services.command_models.exceptions.invalid_course_slug_error import (
    InvalidCourseSlugError,
)


class UnsubscribeCommand(AbstractCommand):
    command_name: Literal["unsubscribe"] = "unsubscribe"
    email: str
    course_slug: str
    organization_id: int

    def execute(self) -> None:
        metric_service = MetricsService()
        try:
            course = Course.objects.get(
                slug=self.course_slug, organization_id=self.organization_id
            )
        except Course.DoesNotExist:
            self.logger.error(
                f"Unsubscribe Failed: Invalid course slug '{self.course_slug}' for organization ID {self.organization_id}"
            )
            raise InvalidCourseSlugError(
                f"Course with slug '{self.course_slug}' does not exist for organization ID {self.organization_id}"
            )

        try:
            learner = Learner.objects.get(
                email=self.email,
                organization_id=self.organization_id,  # type: ignore[misc]
            )
        except Learner.DoesNotExist:
            self.logger.warning(
                f"Unsubscribe Skipped: No learner found with email {self.email}"
            )
            return

        enrollments = Enrollment.objects.filter(learner=learner, course=course).exclude(
            status__in=[EnrollmentStatus.DEACTIVATED, EnrollmentStatus.COMPLETED]
        )
        if not enrollments.exists():
            self.logger.warning(
                f"Unsubscribe Skipped: No active enrollment found for learner {learner.id} in course {course.slug}"
            )
            return

        for enrollment in enrollments:
            for delivery in enrollment.content_deliveries.all():
                delivery.delivery_schedules.filter(
                    status=DeliveryStatus.SCHEDULED
                ).update(status=DeliveryStatus.CANCELED)

        enrollments.update(
            status=EnrollmentStatus.DEACTIVATED,
            deactivation_reason=DeactivationReason.CANCELED,
            final_state_at=timezone.now(),
        )
        metric_service.user_enrollment_deactivated(
            course_slug=course.slug,
            organization_id=course.organization.id,
            reason=DeactivationReason.CANCELED,
        )



from django.urls import path
from django_email_learning.personalised.api.views import (
    QuizSubmissionView,
    SubmitCertificateFormView,
)

app_name = "django_email_learning"

urlpatterns = [
    path("quiz/", QuizSubmissionView.as_view(), name="quiz_submission"),
    path(
        "certificate-form/",
        SubmitCertificateFormView.as_view(),
        name="submit_certificate_form",
    ),
]



from typing import Protocol
from django_email_learning.models import DeactivationReason


class MetricRecorderProtocol(Protocol):
    def user_enrolled_in_course(self, course_slug: str, organization_id: int) -> None:
        ...

    def quiz_sent(self, course_slug: str, organization_id: int, quiz_id: int) -> None:
        ...

    def lesson_sent(
        self, course_slug: str, organization_id: int, lesson_id: int
    ) -> None:
        ...

    def user_enrollment_activated(self, course_slug: str, organization_id: int) -> None:
        ...

    def user_enrollment_deactivated(
        self, course_slug: str, organization_id: int, reason: DeactivationReason
    ) -> None:
        ...

    def user_completed_course(self, course_slug: str, organization_id: int) -> None:
        ...

    def quiz_submitted(
        self, course_slug: str, organization_id: int, quiz_id: int, is_passed: bool
    ) -> None:
        ...

    def method_executed(self, method_name: str, execution_time: int) -> None:
        ...

    def delivery_schedule_blocked(self, content_id: int) -> None:
        ...



from django.views import View
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import ensure_csrf_cookie
from django.db.utils import IntegrityError
from django.db.models.functions import TruncDate
from django.db.models import Count
from django.http import JsonResponse
from django.core.exceptions import ValidationError as DjangoValidationError
from django.db import models, transaction
from django.conf import settings
from django.core.files.storage import default_storage
from django.utils import timezone
from django.urls import reverse
from django.contrib.auth.models import User
from django.contrib.auth.forms import PasswordResetForm
from datetime import timedelta, datetime
from urllib.parse import urlparse
from pydantic import ValidationError
from enum import StrEnum
from django_email_learning.services.command_models.enroll_command import EnrollCommand
from django_email_learning.services.command_models.send_lesson_command import (
    SendLessonCommand,
)
from django_email_learning.services.command_models.verify_enrollment_command import (
    VerifyEnrollmentCommand,
)
from django_email_learning.services.command_models.exceptions.blocked_email_error import (
    BlockedEmailError,
)
from django_email_learning.services.command_models.exceptions.enrollment_already_exists_error import (
    EnrollmentAlreadyExistsError,
)
from django_email_learning.platform.api import serializers
from django_email_learning.platform.api.pagniated_api_mixin import PaginatedApiMixin
from django_email_learning.models import (
    ApiKey,
    Certificate,
    Course,
    CourseContent,
    Enrollment,
    EnrollmentStatus,
    ImapConnection,
    JobExecution,
    JobName,
    Learner,
    OrganizationUser,
    Organization,
)
from django_email_learning.decorators import (
    accessible_for,
    is_an_organization_member,
    is_platform_admin,
)
from typing import Any
import uuid
import json
import logging
import posixpath


logger = logging.getLogger(__name__)

DJANGO_EMAIL_LEARNING_SETTINGS: dict = getattr(settings, "DJANGO_EMAIL_LEARNING", {})


@method_decorator(ensure_csrf_cookie, name="get")
@method_decorator(accessible_for(roles={"admin", "editor"}), name="post")
@method_decorator(accessible_for(roles={"admin", "editor", "viewer"}), name="get")
class CourseView(View):
    def post(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        payload = json.loads(request.body)
        try:
            serializer = serializers.CreateCourseRequest.model_validate(payload)
            course = serializer.to_django_model(
                organization_id=kwargs["organization_id"]
            )
            course.save()
            return JsonResponse(
                serializers.CourseResponse.from_django_model(
                    course, abs_url_builder=request.build_absolute_uri
                ).model_dump(),
                status=201,
            )
        except ValidationError as e:
            return JsonResponse({"error": e.json()}, status=400)
        except (IntegrityError, ValueError) as e:
            return JsonResponse({"error": str(e)}, status=409)

    def get(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        courses = Course.objects.filter(organization_id=kwargs["organization_id"])
        enabled = request.GET.get("enabled")
        if enabled is not None:
            if enabled.lower() in ["true", "yes"]:
                courses = courses.filter(enabled=True)
            elif enabled.lower() in ["false", "no"]:
                courses = courses.filter(enabled=False)

        response_list = []
        for course in courses:
            response_list.append(
                serializers.CourseResponse.from_django_model(
                    course, abs_url_builder=request.build_absolute_uri
                ).model_dump()
            )
        return JsonResponse({"courses": response_list}, status=200)


@method_decorator(accessible_for(roles={"admin", "editor"}), name="post")
@method_decorator(accessible_for(roles={"admin", "editor", "viewer"}), name="get")
class CourseContentView(View):
    def post(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        payload = json.loads(request.body)
        try:
            serializer = serializers.CreateCourseContentRequest.model_validate(payload)
            course = Course.objects.get(id=kwargs["course_id"])
            if serializer.priority is None:
                # Set priority to max existing priority + 1
                max_priority = (
                    CourseContent.objects.filter(course_id=course.id)
                    .aggregate(max_priority=models.Max("priority"))
                    .get("max_priority")
                )
                serializer.priority = (max_priority or 0) + 1
            course_content = serializer.to_django_model(course=course)

            return JsonResponse(
                serializers.CourseContentResponse.model_validate(
                    course_content
                ).model_dump(),
                status=201,
            )
        except Course.DoesNotExist:
            return JsonResponse({"error": "Course not found"}, status=404)
        except ValidationError as e:
            return JsonResponse({"error": e.json()}, status=400)
        except DjangoValidationError as e:
            return JsonResponse({"error": e.messages}, status=400)

    def get(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        try:
            course = Course.objects.get(id=kwargs["course_id"])
            course_contents = course.coursecontent_set.all().order_by("priority")
            response_list = []
            for content in course_contents:
                response_list.append(
                    serializers.CourseContentSummaryResponse.model_validate(
                        content
                    ).model_dump()
                )
            return JsonResponse({"course_contents": response_list}, status=200)
        except Course.DoesNotExist:
            return JsonResponse({"error": "Course not found"}, status=404)


@method_decorator(accessible_for(roles={"admin", "editor"}), name="post")
class ReorderCourseContentView(View):
    def post(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        payload = json.loads(request.body)
        try:
            serializer = serializers.ReorderCourseContentsRequest.model_validate(
                payload
            )
            course = Course.objects.get(id=kwargs["course_id"])
            course_contents = {
                content.id: content for content in course.coursecontent_set.all()
            }

            with transaction.atomic():
                # Collect valid contents and set temporary negative priorities to avoid conflicts
                contents_to_update = []
                for index, content_id in enumerate(serializer.ordered_content_ids):
                    if content_id in course_contents:
                        content = course_contents[content_id]
                        content.priority = -(
                            index + 1
                        )  # Negative priority to avoid unique constraint conflicts
                        contents_to_update.append(content)

                # Bulk update with negative priorities first
                if contents_to_update:
                    CourseContent.objects.bulk_update(contents_to_update, ["priority"])

                    # Now set the final positive priorities
                    for index, content in enumerate(contents_to_update):
                        content.priority = index + 1

                    # Final bulk update with correct priorities
                    CourseContent.objects.bulk_update(contents_to_update, ["priority"])

            return JsonResponse(
                {"message": "Course contents reordered successfully"}, status=200
            )
        except Course.DoesNotExist:
            return JsonResponse({"error": "Course not found"}, status=404)
        except ValidationError as e:
            return JsonResponse({"error": e.json()}, status=400)
        except (IntegrityError, ValueError) as e:
            return JsonResponse({"error": str(e)}, status=409)


@method_decorator(accessible_for(roles={"admin", "editor", "viewer"}), name="get")
@method_decorator(accessible_for(roles={"admin", "editor"}), name="delete")
@method_decorator(accessible_for(roles={"admin", "editor"}), name="post")
class SingleCourseContentView(View):
    def get(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        try:
            course_content = CourseContent.objects.get(id=kwargs["course_content_id"])
            return JsonResponse(
                serializers.CourseContentResponse.model_validate(
                    course_content
                ).model_dump(),
                status=200,
            )
        except CourseContent.DoesNotExist:
            return JsonResponse({"error": "Course content not found"}, status=404)
        except ValidationError as e:
            return JsonResponse({"error": e.json()}, status=400)

    def delete(self, request, *args, **kwargs):  # type: ignore[no-untyped-def]
        try:
            course_content = CourseContent.objects.get(id=kwargs["course_content_id"])
            course_content.delete()
            return JsonResponse(
                {"message": "Course content deleted successfully"}, status=200
            )
        except CourseContent.DoesNotExist:
            return JsonResponse({"error": "Course content not found"}, status=404)
        except ValidationError as e:
            return JsonResponse({"error": e.json()}, status=400)
        except (IntegrityError, ValueError) as e:
            return JsonResponse({"error": str(e)}, status=409)

    def post(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        payload = json.loads(request.body)
        try:
            serializer = serializers.UpdateCourseContentRequest.model_validate(payload)
        except ValidationError as e:
            return JsonResponse({"error": e.json()}, status=400)
        except ValueError as e:
            return JsonResponse({"error": str(e)}, status=400)

        try:
            return self._update_course_content_atomic(
                serializer, kwargs["course_content_id"]
            )
        except CourseContent.DoesNotExist:
            return JsonResponse({"error": "Course content not found"}, status=404)
        except ValidationError as e:
            return JsonResponse({"error": e.json()}, status=400)
        except (IntegrityError, ValueError) as e:
            return JsonResponse({"error": str(e)}, status=409)

    @transaction.atomic
    def _update_course_content_atomic(
        self, serializer: serializers.UpdateCourseContentRequest, course_content_id: int
    ) -> JsonResponse:
        course_content = CourseContent.objects.get(id=course_content_id)

        if serializer.priority is not None:
            course_content.priority = serializer.priority
        if serializer.waiting_period is not None:
            course_content.waiting_period = serializer.waiting_period.to_seconds()

        if serializer.is_published is not None:
            course_content.is_published = serializer.is_published
            course_content.save()

        if serializer.lesson is not None and course_content.lesson is not None:
            lesson_serializer = serializer.lesson
            lesson = course_content.lesson
            if lesson_serializer.title is not None:
                lesson.title = lesson_serializer.title
            if lesson_serializer.content is not None:
                lesson.content = lesson_serializer.content
            lesson.save()

        if serializer.quiz is not None and course_content.quiz is not None:
            quiz_serializer = serializer.quiz
            quiz = course_content.quiz
            if quiz_serializer.title is not None:
                quiz.title = quiz_serializer.title
            if quiz_serializer.required_score is not None:
                quiz.required_score = quiz_serializer.required_score
            if quiz_serializer.selection_strategy is not None:
                quiz.selection_strategy = quiz_serializer.selection_strategy.value
            if quiz_serializer.deadline_days is not None:
                quiz.deadline_days = quiz_serializer.deadline_days
            if quiz_serializer.questions is not None:
                question_ids = set()
                for question_data in quiz_serializer.questions:
                    if question_data.id:
                        question = quiz.questions.get(id=question_data.id)
                        question.text = question_data.text
                        question.priority = question_data.priority
                        question.save()
                    else:
                        question = quiz.questions.create(
                            text=question_data.text, priority=question_data.priority
                        )
                    question_ids.add(question.id)
                    answer_ids = set()
                    for answer_data in question_data.answers:
                        if answer_data.id:
                            answer = question.answers.get(id=answer_data.id)
                            answer.text = answer_data.text
                            answer.is_correct = answer_data.is_correct
                            answer.save()
                        else:
                            answer = question.answers.create(
                                text=answer_data.text, is_correct=answer_data.is_correct
                            )
                        answer_ids.add(answer.id)
                    question.answers.exclude(
                        id__in=answer_ids
                    ).delete()  # Remove any answers that were not included in the update payload
                quiz.questions.exclude(
                    id__in=question_ids
                ).delete()  # Remove any questions that were not included in the update payload
            quiz.save()

        course_content.save()
        return JsonResponse(
            serializers.CourseContentResponse.model_validate(
                course_content
            ).model_dump(),
            status=200,
        )


@method_decorator(accessible_for(roles={"admin", "editor"}), name="post")
@method_decorator(accessible_for(roles={"admin", "editor"}), name="delete")
@method_decorator(accessible_for(roles={"admin", "editor", "viewer"}), name="get")
class SingleCourseView(View):
    def get(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        try:
            course = Course.objects.get(id=kwargs["course_id"])
            return JsonResponse(
                serializers.CourseResponse.from_django_model(
                    course, abs_url_builder=request.build_absolute_uri
                ).model_dump(),
                status=200,
            )
        except Course.DoesNotExist:
            return JsonResponse({"error": "Course not found"}, status=404)
        except ValidationError as e:
            return JsonResponse({"error": e.json()}, status=400)
        except (IntegrityError, ValueError) as e:
            return JsonResponse({"error": str(e)}, status=409)

    def post(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        payload = json.loads(request.body)
        try:
            serializer = serializers.UpdateCourseRequest.model_validate(payload)
            course = serializer.to_django_model(course_id=kwargs["course_id"])
            course.save()
            return JsonResponse(
                serializers.CourseResponse.from_django_model(
                    course, abs_url_builder=request.build_absolute_uri
                ).model_dump(),
                status=200,
            )
        except ValidationError as e:
            return JsonResponse({"error": e.json()}, status=400)
        except (IntegrityError, ValueError) as e:
            return JsonResponse({"error": str(e)}, status=409)

    def delete(self, request, *args, **kwargs):  # type: ignore[no-untyped-def]
        try:
            course = Course.objects.get(id=kwargs["course_id"])
            course.delete()
            return JsonResponse({"message": "Course deleted successfully"}, status=200)
        except Course.DoesNotExist:
            return JsonResponse({"error": "Course not found"}, status=404)
        except ValidationError as e:
            return JsonResponse({"error": e.json()}, status=400)
        except (IntegrityError, ValueError) as e:
            return JsonResponse({"error": str(e)}, status=409)


class GoogleAuthSession(View):
    def get(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        state = str(uuid.uuid4())
        request.session["google_oauth_state"] = state
        return JsonResponse({"state": state}, status=200)


@method_decorator(accessible_for(roles={"admin", "editor"}), name="post")
@method_decorator(accessible_for(roles={"admin", "editor", "viewer"}), name="get")
class ImapConnectionView(View):
    def get(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        response_list = []
        imap_connections = ImapConnection.objects.filter(
            organization_id=kwargs["organization_id"]
        )
        for connection in imap_connections:
            response_list.append(
                serializers.ImapConnectionResponse.model_validate(
                    connection
                ).model_dump()
            )
        return JsonResponse({"imap_connections": response_list}, status=200)

    def post(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        payload = json.loads(request.body)
        try:
            serializer = serializers.CreateImapConnectionRequest.model_validate(payload)
            imap_connection = serializer.to_django_model(
                organization_id=kwargs["organization_id"]
            )
            imap_connection.save()
            return JsonResponse(
                serializers.ImapConnectionResponse.model_validate(
                    imap_connection
                ).model_dump(),
                status=201,
            )
        except ValidationError as e:
            return JsonResponse({"error": e.json()}, status=400)
        except IntegrityError as e:
            return JsonResponse({"error": str(e)}, status=409)


@method_decorator(ensure_csrf_cookie, name="get")
@method_decorator(is_an_organization_member(), name="get")
@method_decorator(is_platform_admin(), name="post")
class OrganizationsView(View):
    def get(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        if request.user.is_superuser:
            organizations = Organization.objects.all()
        else:
            organizations_users = OrganizationUser.objects.select_related(
                "organization"
            ).filter(user_id=request.user.id)
            organizations = [ou.organization for ou in organizations_users]  # type: ignore[assignment]
        response_list = []
        for org in organizations:
            response_list.append(
                serializers.OrganizationResponse.from_django_model(
                    org, request.build_absolute_uri
                ).model_dump()
            )
        return JsonResponse({"organizations": response_list}, status=200)

    def post(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        try:
            payload = json.loads(request.body)
            serializer = serializers.CreateOrganizationRequest.model_validate(payload)
            organization = serializer.to_django_model()
            organization.save()
            # Add the creating user as an admin of the organization
            org_user = OrganizationUser(
                user_id=request.user.id, organization_id=organization.id, role="admin"
            )
            org_user.save()
            return JsonResponse(
                serializers.OrganizationResponse.from_django_model(
                    organization,
                    request.build_absolute_uri,
                ).model_dump(),
                status=201,
            )
        except ValidationError as e:
            return JsonResponse({"error": e.json()}, status=400)
        except IntegrityError as e:
            return JsonResponse({"error": str(e)}, status=409)


@method_decorator(accessible_for(roles={"admin"}), name="post")
@method_decorator(accessible_for(roles={"admin"}), name="get")
class OrganizationUsersView(View):
    def post(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        try:
            payload = json.loads(request.body)
            serializer = serializers.AddOrganizationUserRequest.model_validate(payload)
            organization = Organization.objects.get(id=kwargs["organization_id"])
            org_user = OrganizationUser(
                user_id=serializer.user_id,
                organization=organization,
                role=serializer.role,
            )
            org_user.save()
            return JsonResponse(
                serializers.OrganizationUserResponse.from_django_model(
                    org_user
                ).model_dump(),
                status=201,
            )
        except Organization.DoesNotExist:
            return JsonResponse({"error": "Organization not found"}, status=404)
        except ValidationError as e:
            return JsonResponse({"error": e.json()}, status=400)
        except IntegrityError as e:
            return JsonResponse({"error": str(e)}, status=409)

    def get(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        organization_users = OrganizationUser.objects.filter(
            organization_id=kwargs["organization_id"]
        )
        response_list = []
        for org_user in organization_users:
            response_list.append(
                serializers.OrganizationUserResponse.from_django_model(
                    org_user
                ).model_dump()
            )
        return JsonResponse({"organization_users": response_list}, status=200)


@method_decorator(accessible_for(roles={"admin"}), name="delete")
class SingleOrganizationUserView(View):
    def delete(self, request, *args, **kwargs):  # type: ignore[no-untyped-def]
        try:
            org_user = OrganizationUser.objects.get(id=kwargs["user_id"])
            org_user.delete()
            return JsonResponse(
                {"message": "Organization user removed successfully"}, status=200
            )
        except OrganizationUser.DoesNotExist:
            return JsonResponse({"error": "Organization user not found"}, status=404)
        except ValidationError as e:
            return JsonResponse({"error": e.json()}, status=400)
        except IntegrityError as e:
            return JsonResponse({"error": str(e)}, status=409)

    def post(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        try:
            payload = json.loads(request.body)
            serializer = serializers.UpdateOrganizationUserRoleRequest.model_validate(
                payload
            )
            org_user = OrganizationUser.objects.get(
                organization_id=kwargs["organization_id"], user_id=kwargs["user_id"]
            )
            org_user.role = serializer.role
            org_user.save()
            return JsonResponse(
                serializers.OrganizationUserResponse.from_django_model(
                    org_user
                ).model_dump(),
                status=200,
            )
        except OrganizationUser.DoesNotExist:
            return JsonResponse({"error": "Organization user not found"}, status=404)
        except ValidationError as e:
            return JsonResponse({"error": e.json()}, status=400)
        except IntegrityError as e:
            return JsonResponse({"error": str(e)}, status=409)


@method_decorator(is_platform_admin(), name="post")
@method_decorator(is_platform_admin(), name="delete")
@method_decorator(accessible_for(roles={"admin", "editor", "viewer"}), name="get")
class SingleOrganizationView(View):
    def post(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        try:
            payload = json.loads(request.body)
            serializer = serializers.UpdateOrganizationRequest.model_validate(payload)
            organization = Organization.objects.get(id=kwargs["organization_id"])
            if serializer.name is not None:
                organization.name = serializer.name
            if serializer.description is not None:
                organization.description = serializer.description
            if serializer.logo is not None:
                organization.logo = serializer.logo
            if serializer.remove_logo:
                organization.logo = None
            organization.save()
            return JsonResponse(
                serializers.OrganizationResponse.from_django_model(
                    organization,
                    request.build_absolute_uri,
                ).model_dump(),
                status=200,
            )
        except Organization.DoesNotExist:
            return JsonResponse({"error": "Organization not found"}, status=404)
        except ValidationError as e:
            return JsonResponse({"error": e.json()}, status=400)
        except IntegrityError as e:
            return JsonResponse({"error": str(e)}, status=409)

    def delete(self, request, *args, **kwargs):  # type: ignore[no-untyped-def]
        try:
            organization = Organization.objects.get(id=kwargs["organization_id"])
            organization.delete()
            return JsonResponse(
                {"message": "Organization deleted successfully"}, status=200
            )
        except Organization.DoesNotExist:
            return JsonResponse({"error": "Organization not found"}, status=404)
        except ValidationError as e:
            return JsonResponse({"error": e.json()}, status=400)
        except IntegrityError as e:
            return JsonResponse({"error": str(e)}, status=409)

    def get(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        try:
            organization = Organization.objects.get(id=kwargs["organization_id"])
            return JsonResponse(
                serializers.OrganizationResponse.from_django_model(
                    organization,
                    request.build_absolute_uri,
                ).model_dump(),
                status=200,
            )
        except Organization.DoesNotExist:
            return JsonResponse({"error": "Organization not found"}, status=404)
        except ValidationError as e:
            return JsonResponse({"error": e.json()}, status=400)


@method_decorator((is_an_organization_member(only_admin=True)), name="post")
class GetOrCreateUserByEmail(View):
    def post(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        payload = json.loads(request.body)
        serializer = serializers.GetOrCreateUserRequest.model_validate(payload)
        try:
            email = serializer.email
            organization_id = serializer.organization_id
            user = User.objects.filter(email=email).first()
            if not user:
                user = User.objects.create_user(
                    username=email, email=email, password=uuid.uuid4().hex
                )
                form = PasswordResetForm(data={"email": email})
                if form.is_valid():
                    form.save(
                        request=request,
                        use_https=request.is_secure(),
                        from_email=DJANGO_EMAIL_LEARNING_SETTINGS.get(
                            "FROM_EMAIL", settings.DEFAULT_FROM_EMAIL
                        ),
                        email_template_name="emails/password_reset.txt",
                        html_email_template_name="emails/password_reset.html",
                        extra_email_context={
                            "organization": Organization.objects.get(
                                id=organization_id
                            ).name
                        },
                    )
                else:
                    raise ValueError(
                        "Failed to send password reset email to the new user."
                    )
            return JsonResponse(
                serializers.UserResponse.model_validate(user).model_dump(), status=200
            )
        except ValidationError as e:
            return JsonResponse({"error": e.json()}, status=400)
        except IntegrityError as e:
            return JsonResponse({"error": str(e)}, status=409)


@method_decorator(accessible_for(roles={"admin", "editor"}), name="post")
class SendLessonToPlatformUser(View):
    def post(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        if not request.user.email:
            return JsonResponse(
                {"error": "User does not have an email address"}, status=400
            )
        payload = json.loads(request.body)
        serializer = serializers.Identifier.model_validate(payload)
        email = request.user.email
        try:
            # Accept course content id directly (used by platform content table),
            # with a lesson-id fallback for backward compatibility.
            course_content = CourseContent.objects.filter(
                id=serializer.id,
                type="lesson",
                course__organization_id=kwargs["organization_id"],
            ).first()
            if course_content is None:
                course_content = CourseContent.objects.get(
                    lesson_id=serializer.id,
                    course__organization_id=kwargs["organization_id"],
                )
            SendLessonCommand(content_id=course_content.id, email=email).execute()
        except CourseContent.DoesNotExist:
            return JsonResponse({"error": "Lesson not found"}, status=404)
        return JsonResponse(
            {"message": "Email content logged successfully"}, status=200
        )


@method_decorator(accessible_for(roles={"admin", "editor"}), name="post")
@method_decorator(accessible_for(roles={"admin", "editor"}), name="delete")
class FileView(View):
    def post(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        uploaded_file = request.FILES.get("file")
        if not uploaded_file:
            return JsonResponse({"error": "No file uploaded"}, status=400)

        # check file extension
        allowed_extensions = ["png", "jpg", "jpeg", "svg"]
        file_extension = uploaded_file.name.split(".")[-1].lower()
        if file_extension not in allowed_extensions:
            return JsonResponse({"error": "Invalid file type"}, status=400)

        date_prefix = timezone.now().strftime("%Y%m%d")

        file_path = default_storage.save(
            f"uploads/{date_prefix}/{kwargs['organization_id']}/{uploaded_file.name}",
            uploaded_file,
        )
        file_url = default_storage.url(file_path)
        return JsonResponse({"file_url": file_url, "file_path": file_path}, status=201)

    def delete(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        try:
            payload = json.loads(request.body or "{}")
        except json.JSONDecodeError:
            return JsonResponse({"error": "Invalid request body"}, status=400)

        file_path = payload.get("file_path")
        file_url = payload.get("file_url")

        if not file_path and file_url:
            parsed_url_path = urlparse(file_url).path
            media_url = settings.MEDIA_URL or "/media/"
            normalized_media_url = (
                media_url if media_url.endswith("/") else f"{media_url}/"
            )
            if parsed_url_path.startswith(normalized_media_url):
                file_path = parsed_url_path[len(normalized_media_url) :]

        if not file_path:
            return JsonResponse({"error": "file_path is required"}, status=400)

        normalized_file_path = posixpath.normpath(str(file_path)).lstrip("/")
        path_parts = normalized_file_path.split("/")
        organization_id = str(kwargs["organization_id"])

        if (
            len(path_parts) < 4
            or path_parts[0] != "uploads"
            or path_parts[2] != organization_id
            or ".." in path_parts
        ):
            return JsonResponse({"error": "Invalid file path"}, status=400)

        if not default_storage.exists(normalized_file_path):
            return JsonResponse({"error": "File not found"}, status=404)

        default_storage.delete(normalized_file_path)
        return JsonResponse({"message": "File deleted successfully"}, status=200)


@method_decorator(is_an_organization_member(), name="post")
class UpdateSessionView(View):
    def post(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        try:
            payload = json.loads(request.body)
            serializer = serializers.UpdateSessionRequest.model_validate(payload)
            organization_id = serializer.active_organization_id
        except ValidationError as e:
            return JsonResponse({"error": e.json()}, status=400)

        if (
            not OrganizationUser.objects.filter(
                user_id=request.user.id, organization_id=organization_id
            ).exists()
            and not request.user.is_superuser
        ):
            return JsonResponse(
                {"error": "Not a valid organization for the user."}, status=409
            )
        request.session["active_organization_id"] = organization_id
        response_serializer = serializers.SessionInfo.populate_from_session(
            request.session
        )
        return JsonResponse(response_serializer.model_dump(), status=200)


@method_decorator(accessible_for(roles={"admin", "editor", "viewer"}), name="get")
class LearnersView(PaginatedApiMixin, View):
    def get_query_set(self, request: Any) -> models.QuerySet:
        organization_id = self.kwargs["organization_id"]
        qs = Enrollment.objects.filter(course__organization_id=organization_id)
        if "course_id" in request.GET:
            course_id = request.GET["course_id"]
            qs = qs.filter(course_id=course_id)
        if "is_active" in request.GET:
            is_active_str = request.GET["is_active"].lower()
            if is_active_str in ["true", "yes"]:
                qs = qs.filter(status=EnrollmentStatus.ACTIVE)
        if "search" in request.GET:
            search_term = request.GET["search"]
            qs = qs.filter(models.Q(learner__email__icontains=search_term))
        learner_ids = qs.values("learner_id").distinct()
        return Learner.objects.filter(id__in=learner_ids)

    def get_item_serializer_class(self) -> Any:
        return serializers.LearnerResponse


@method_decorator(accessible_for(roles={"admin", "editor", "viewer"}), name="get")
class SingleLearnerView(View):
    def get(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        try:
            learner = Learner.objects.get(id=kwargs["learner_id"])
            enrollments = Enrollment.objects.filter(learner=learner)
            enroolments_list = []
            for enrollment in enrollments:
                certificate = Certificate.objects.filter(enrollment=enrollment).first()
                certificate_url = None
                if certificate:
                    certificate_url = request.build_absolute_uri(
                        reverse(
                            "django_email_learning:personalised:certificate",
                            kwargs={
                                "certificate_number": certificate.certificate_number
                            },
                        )
                    )
                enroolments_list.append(
                    serializers.EnrollmentSummaryResponse(
                        id=enrollment.id,
                        course_title=enrollment.course.title,
                        status=EnrollmentStatus(enrollment.status),
                        progress=enrollment.progress_percentage,
                        certificate_url=certificate_url,
                    )
                )
            return JsonResponse(
                serializers.LearnerDetailResponse(
                    id=learner.id, email=learner.email, enrollments=enroolments_list
                ).model_dump(),
                status=200,
            )
        except Learner.DoesNotExist:
            return JsonResponse({"error": "Learner not found"}, status=404)
        except ValidationError as e:
            logger.error(f"Error in SingleLearnerView: {e.json()}")
            return JsonResponse({"error": "An internal error occurred."}, status=500)


@method_decorator(accessible_for(roles={"admin"}), name="post")
class EnrollmentsView(PaginatedApiMixin, View):
    def post(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        payload = json.loads(request.body)
        try:
            serializer = serializers.CreateEnrollmentRequest.model_validate(payload)
            try:
                course = Course.objects.get(
                    id=kwargs["course_id"], organization_id=kwargs["organization_id"]
                )
            except Course.DoesNotExist:
                return JsonResponse({"error": "Course not found"}, status=404)
            command = EnrollCommand(
                email=serializer.learner_email,
                course_slug=course.slug,
                organization_id=kwargs["organization_id"],
                no_verification=True,  # skip verification email for manual enrollments through the API
            )
            try:
                command.execute()
            except BlockedEmailError as e:
                return JsonResponse({"error": str(e)}, status=403)
            except EnrollmentAlreadyExistsError as e:
                return JsonResponse({"error": str(e)}, status=409)

            enrollment = Enrollment.objects.get(
                learner__email=serializer.learner_email, course_id=kwargs["course_id"]
            )
            verify_command = VerifyEnrollmentCommand(
                enrollment_id=enrollment.id,
                verification_code=enrollment.activation_code,  # type: ignore[arg-type]
            )
            verify_command.execute()
            enrollment.refresh_from_db()
            return JsonResponse(
                serializers.EnrollmentResponse.from_django_model(
                    enrollment
                ).model_dump(),
                status=201,
            )
        except ValidationError as e:
            return JsonResponse({"error": e.json()}, status=400)


@method_decorator(accessible_for(roles={"admin", "editor", "viewer"}), name="get")
class EnrollmentView(View):
    def get(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        try:
            enrollment = Enrollment.objects.get(id=kwargs["enrollment_id"])
            return JsonResponse(
                serializers.EnrollmentResponse.from_django_model(
                    enrollment
                ).model_dump(),
                status=200,
            )
        except Enrollment.DoesNotExist:
            return JsonResponse({"error": "Enrollment not found"}, status=404)
        except ValidationError as e:
            return JsonResponse({"error": e.json()}, status=400)


@method_decorator(accessible_for(roles={"admin", "editor", "viewer"}), name="get")
class EnrollmentsStatisticsView(View):
    def get(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        course_id = kwargs["course_id"]
        a_week_ago = timezone.now() - timedelta(days=7)
        enrollments = (
            Enrollment.objects.filter(course_id=course_id, enrolled_at__gte=a_week_ago)
            .annotate(created_date=TruncDate("enrolled_at"))
            .values(
                "created_date",
            )
            .annotate(count=Count("id"))
            .order_by("created_date")
        )
        dates = [a_week_ago.date() + timedelta(days=i) for i in range(8)]
        enrollments_dict = {
            enrollment["created_date"]: enrollment["count"]
            for enrollment in enrollments
        }
        stats = [
            {"date": date.isoformat(), "count": enrollments_dict.get(date, 0)}
            for date in dates
        ]
        return JsonResponse({"statistics": stats}, status=200)


@method_decorator(is_platform_admin(), name="post")
@method_decorator(is_platform_admin(), name="get")
class ApiKeyView(View):
    def post(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        try:
            key = ApiKey.generate_key()
            api_key = ApiKey(key=key, created_by=request.user)
            api_key.save()
            return JsonResponse(
                serializers.ApiKeyResponse.from_django_model(api_key).model_dump(),
                status=201,
            )
        except ValidationError as e:
            return JsonResponse({"error": e.json()}, status=400)
        except IntegrityError as e:
            return JsonResponse({"error": str(e)}, status=409)

    def get(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        api_keys = ApiKey.objects.all()  # type: ignore[attr-defined]
        response_list = []
        for api_key in api_keys:
            response_list.append(
                serializers.ApiKeyResponse.from_django_model(api_key).model_dump()
            )
        return JsonResponse({"api_keys": response_list}, status=200)


@method_decorator(is_platform_admin(), name="delete")
class SingleApiKeyView(View):
    def delete(self, request, *args, **kwargs):  # type: ignore[no-untyped-def]
        try:
            api_key = ApiKey.objects.get(id=kwargs["api_key_id"])
            api_key.delete()
            return JsonResponse({"message": "API Key deleted successfully"}, status=200)
        except ApiKey.DoesNotExist:
            return JsonResponse({"error": "API Key not found"}, status=404)
        except ValidationError as e:
            return JsonResponse({"error": e.json()}, status=400)
        except IntegrityError as e:
            return JsonResponse({"error": str(e)}, status=409)


class JobHealthStatus(StrEnum):
    SUCCESS = "healthy"
    WARNING = "warning"
    CRITICAL = "critical"


DEFAULT_SUCCESS_THRESHOLD_MINUTES = 15
DEFAULT_WARNING_THRESHOLD_MINUTES = 45


@method_decorator(is_an_organization_member(), name="get")
class JobsStatus(View):
    def get(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        jobs_status = {}
        for job in JobName:
            last_execution = (
                JobExecution.objects.filter(job_name=job.value)
                .order_by("-started_at")
                .first()
            )
            jobs_status[job.value] = {
                "job_name": job.value,
                "last_execution_status": last_execution.status
                if last_execution
                else None,
                "last_execution_started_at": last_execution.started_at.isoformat()
                if last_execution
                else None,
                "last_execution_finished_at": last_execution.finished_at.isoformat()
                if last_execution and last_execution.finished_at
                else None,
                "job_health_status": self.calculate_job_health_status(
                    last_execution.started_at
                )
                if last_execution
                else JobHealthStatus.CRITICAL.value,
            }

        return JsonResponse({"jobs": jobs_status}, status=200)

    @staticmethod
    def calculate_job_health_status(last_execution_started_at: datetime) -> str:
        success_threshold = DJANGO_EMAIL_LEARNING_SETTINGS.get(
            "JOB_HEALTH_SUCCESS_THRESHOLD_MINUTES", DEFAULT_SUCCESS_THRESHOLD_MINUTES
        )
        warning_threshold = DJANGO_EMAIL_LEARNING_SETTINGS.get(
            "JOB_HEALTH_WARNING_THRESHOLD_MINUTES", DEFAULT_WARNING_THRESHOLD_MINUTES
        )
        if not isinstance(success_threshold, int) or success_threshold <= 0:
            success_threshold = DEFAULT_SUCCESS_THRESHOLD_MINUTES
        if not isinstance(warning_threshold, int) or warning_threshold <= 0:
            warning_threshold = DEFAULT_WARNING_THRESHOLD_MINUTES
        if warning_threshold <= success_threshold:
            warning_threshold = (
                success_threshold + 30
            )  # Ensure warning threshold is greater than success threshold
        now = timezone.now()
        time_diff = now - last_execution_started_at
        minutes_diff = time_diff.total_seconds() / 60
        if minutes_diff <= success_threshold:
            return JobHealthStatus.SUCCESS.value
        elif minutes_diff <= warning_threshold:
            return JobHealthStatus.WARNING.value
        else:
            return JobHealthStatus.CRITICAL.value


class RootView(View):
    def get(self, request, *args, **kwargs) -> JsonResponse:  # type: ignore[no-untyped-def]
        return JsonResponse({"message": "Email Learning API is running."}, status=200)



from abc import ABC, abstractmethod
from pydantic import BaseModel
import logging


class AbstractCommand(ABC, BaseModel):
    @property
    def logger(self) -> logging.Logger:
        return logging.getLogger(self.__class__.__name__)

    @abstractmethod
    def execute(self) -> None:
        raise NotImplementedError("Subclasses must implement the execute method")



from django.contrib.auth.models import Group, Permission
from django.contrib.contenttypes.models import ContentType
from django.db.models.signals import post_migrate
from django.dispatch import receiver
from django_email_learning.apps import PLATFORM_ADMIN_GROUP_NAME


@receiver(post_migrate)
def create_platform_admin_group(sender, **kwargs) -> None:  # type: ignore[no-untyped-def]
    if sender.name != "django_email_learning":
        return

    organization_ct = ContentType.objects.get(
        app_label="django_email_learning", model="organization"
    )
    organization_user_ct = ContentType.objects.get(
        app_label="django_email_learning", model="organizationuser"
    )
    course_ct = ContentType.objects.get(
        app_label="django_email_learning", model="course"
    )
    imap_connection_ct = ContentType.objects.get(
        app_label="django_email_learning", model="imapconnection"
    )
    lesson_ct = ContentType.objects.get(
        app_label="django_email_learning", model="lesson"
    )
    quiz_ct = ContentType.objects.get(app_label="django_email_learning", model="quiz")
    question_ct = ContentType.objects.get(
        app_label="django_email_learning", model="question"
    )
    answer_ct = ContentType.objects.get(
        app_label="django_email_learning", model="answer"
    )
    course_content_ct = ContentType.objects.get(
        app_label="django_email_learning", model="coursecontent"
    )

    perms = Permission.objects.filter(
        content_type__in=[
            organization_ct,
            organization_user_ct,
            course_ct,
            imap_connection_ct,
            lesson_ct,
            quiz_ct,
            question_ct,
            answer_ct,
            course_content_ct,
        ],
        codename__in=[
            "add_organization",
            "change_organization",
            "delete_organization",
            "add_organizationuser",
            "change_organizationuser",
            "delete_organizationuser",
            "add_course",
            "change_course",
            "delete_course",
            "add_imapconnection",
            "change_imapconnection",
            "delete_imapconnection",
            "add_lesson",
            "change_lesson",
            "delete_lesson",
            "add_quiz",
            "change_quiz",
            "delete_quiz",
            "add_question",
            "change_question",
            "delete_question",
            "add_answer",
            "change_answer",
            "delete_answer",
            "add_coursecontent",
            "change_coursecontent",
            "delete_coursecontent",
        ],
    )

    platform_admin_group, created = Group.objects.get_or_create(
        name=PLATFORM_ADMIN_GROUP_NAME
    )
    platform_admin_group.permissions.set(perms)
    print(f"{PLATFORM_ADMIN_GROUP_NAME} group created.")


@receiver(post_migrate)
def create_default_organization(sender, **kwargs):  # type: ignore[no-untyped-def]
    if sender.name != "django_email_learning":
        return

    Organization = sender.get_model("Organization")
    if not Organization.objects.count():
        organization = Organization.objects.create(name="My Organization")
        organization.save()
        print("Default organization 'My Organization' created.")



from django.apps import AppConfig
from django.core import checks


PLATFORM_ADMIN_GROUP_NAME = "Platform Admin"


def check_site_base_url_config(app_configs, **kwargs):  # type: ignore[no-untyped-def]
    errors = []
    from django.conf import settings

    if (
        not hasattr(settings, "DJANGO_EMAIL_LEARNING")
        or "SITE_BASE_URL" not in settings.DJANGO_EMAIL_LEARNING
    ):
        errors.append(
            checks.Error(
                "DJANGO_EMAIL_LEARNING['SITE_BASE_URL'] is not set in settings.",
                hint="Please set DJANGO_EMAIL_LEARNING['SITE_BASE_URL'] to the base URL of your site.",
                id="django_email_learning.E001",
            )
        )
    if (
        not hasattr(settings, "DJANGO_EMAIL_LEARNING")
        or "ENCRYPTION_SECRET_KEY" not in settings.DJANGO_EMAIL_LEARNING
    ):
        errors.append(
            checks.Error(
                "DJANGO_EMAIL_LEARNING['ENCRYPTION_SECRET_KEY'] is not set in settings.",
                hint="Please set DJANGO_EMAIL_LEARNING['ENCRYPTION_SECRET_KEY'] to a long, random string.",
                id="django_email_learning.E002",
            )
        )
    return errors


class EmailLearningConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "django_email_learning"
    verbose_name = "Email Learning"

    def ready(self) -> None:
        import django_email_learning.signals  # noqa

        checks.register(check_site_base_url_config)
